pub fn validate_unused_kinds_audit(
lang: &dyn Language,
documented_unused: &[&str],
) -> Result<(), String>Expand description
Validate that a language’s unused node kinds audit is complete and accurate.
This function checks:
- All kinds in
documented_unusedactually exist in the grammar - All potentially useful kinds from the grammar are either used or documented
Call this from each language’s unused_node_kinds_audit test.