pub struct MatlabModuleResolver;Expand description
Module resolver for MATLAB.
In MATLAB, one function or class lives in each .m file, and the filename
is the function/class name. Functions are found on the path — there is no
explicit import syntax (the import command is for Java class imports, not
MATLAB function lookup). ImportSpec.raw will be a function/class name.