support_for_path

Function support_for_path 

pub fn support_for_path(path: &Path) -> Option<&'static dyn Language>
Get language support from a file path.

Returns None if the file has no extension, the extension is not recognized, or the feature is not enabled.